package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.time.LocalDateTime;
import java.util.List;

import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.entity.TaIssueProcess;
import com.example.civilizedcity.service.TaAttachService;
import com.example.civilizedcity.service.TaIssueProcessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.service.TaIssueService;

 /**
 * 问题表;(ta_issue)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "问题表对象功能接口")
@RestController
@RequestMapping("/")
public class TaIssueController extends BaseController {
    
    @Autowired
    private TaIssueService taIssueService;

    @Autowired
    private TaAttachService taAttachService;

    @Autowired
    private TaIssueProcessService taIssueProcessService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param issueId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIssue/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taIssueService.getById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taIssue")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {

        SysUser sysUser = currentUser();
        String orgId = Constants.ROOT_ORG.equals(sysUser.getOrgId()) ? null : sysUser.getOrgId();

        IPage<TaIssue> pg = new Page<>(pageNum, pageSize);
        // QueryWrapper<TaIssue> queryWrapper = new QueryWrapper<>();
        // queryWrapper.orderByDesc("create_date");
        IPage<TaIssue> result = taIssueService.getPageBy(pg, orgId);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taIssue")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaIssue taIssue) throws Exception {
        SysUser user = currentUser();

        taIssue.setIssueId(null);
        taIssue.setOrgId(null); // 暂时没有责任单位
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessStatus(Constants.PROCESS_STATUS_READY);
        taIssue.setProcessNum(1);
        taIssue.setSourceType(Constants.ISSUE_SOURCE_INSPECTOR);
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreatePerson(user.getUserId());
        taIssue.setPersonName(user.getName());
        taIssue.setCreateDate(LocalDateTime.now());
        taIssueService.save(taIssue);

        // 保存文件
        List<TaAttach> attachList = taIssue.getAttachList();
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                item.setAttachId(null);
                item.setOwnerType(Constants.ATTACH_SOURCE_ISSUE);
                item.setOwnerId(taIssue.getIssueId().toString());
                item.setStatus(Constants.STATUS_NORMAL);
                item.setCreateDate(LocalDateTime.now());
            }
        }

        taAttachService.saveBatch(attachList);

        // 流程开始
        TaIssueProcess process = new TaIssueProcess();
        process.setIssueId(taIssue.getIssueId());
        process.setOrgId(user.getOrgId());
        process.setProcessNode(Constants.PROCESS_START);
        process.setProcessStatus(Constants.PROCESS_STATUS_READY);
        process.setStatus(Constants.STATUS_NORMAL);
        process.setCreateUser(user.getUserId());
        process.setUserName(user.getName());
        process.setCreateDate(LocalDateTime.now());
        taIssueProcessService.save(process);

        return ResponseBean.success(taIssue);
    }
    
    /** 
     * 更新数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taIssue/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaIssue taIssue,
                            @ApiParam("对象ID") @PathVariable Integer id ) throws Exception {
        taIssueService.updateById(taIssue);
        return ResponseBean.success(taIssue);
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param issueId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taIssue/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id){
        taIssueService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}